﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Bl.CsharPlagger.Plugin.File;
using Bl.CsharPlagger.Engine.Config;
using System.Collections;
using Bl.CsharPlagger;
using Bl.CsharPlagger.Engine.Util;
using Bl.CsharPlagger.Engine.Model;
using System.Xml;
using Bl.CsharPlagger.Engine.Exceptions;
using log4net;
using System.Reflection;
using System.Diagnostics;
using Bl.CsharPlagger.Win.Properties;
using Bl.DBee.Context;
using Bl.CsharPlagger.Engine;

namespace Bl.CsharPlagger.Win
{
    public partial class FormPlagger : Form
    {
        private static readonly ILog logger = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        public FormPlagger()
        {
            InitializeComponent();

            toolStripStatusLabel1.Text = "";
            toolStripStatusLabel2.Text = "";
            tsmSaveConfig.Enabled = false;

            ArrayList list = Settings.Default.ConfigList;
            if (list == null)
                list = new ArrayList();
            foreach (string item in list)
            {
                cmbConfig.Items.Add(item);
            }
        }

        /// <summary>
        /// メニューの設定ファイル読込み
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmOpenConfig_Click(object sender, EventArgs e)
        {
            try
            {
                OpenConfigFile();
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// メニューのテンプレート作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmTemplate_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog sfd = new SaveFileDialog();

                sfd.FileName = "設定ファイルテンプレート.xml";
                sfd.InitialDirectory = @"";
                sfd.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
                sfd.FilterIndex = 1;
                sfd.Title = "名前を付けてテンプレートの保存";
                sfd.RestoreDirectory = true;
                sfd.OverwritePrompt = true;
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    TemplateUtil.WriteTemplate(sfd.FileName);
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }

        }

        /// <summary>
        /// メニューのドキュメント用スキーマ作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmScheam_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog sfd = new SaveFileDialog();

                sfd.FileName = "スキーマ.xml";
                sfd.InitialDirectory = @"";
                sfd.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
                sfd.FilterIndex = 1;
                sfd.Title = "名前を付けてスキーマの保存";
                sfd.RestoreDirectory = true;
                sfd.OverwritePrompt = true;
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    TemplateUtil.WritePluginScheam(sfd.FileName);
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// メニューのXMLスキーマ作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmXmlScheam_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog sfd = new SaveFileDialog();

                sfd.FileName = "スキーマ.xml";
                sfd.InitialDirectory = @"";
                sfd.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
                sfd.FilterIndex = 1;
                sfd.Title = "名前を付けてスキーマの保存";
                sfd.RestoreDirectory = true;
                sfd.OverwritePrompt = true;
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    TemplateUtil.WritePluginScheam(sfd.FileName);
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        ///  メニューのTextスキーマ作成
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmTextScheam_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog sfd = new SaveFileDialog();

                sfd.FileName = "スキーマ.txt";
                sfd.InitialDirectory = @"";
                sfd.Filter = "Textファイル(*.txt)|*.txt|すべてのファイル(*.*)|*.*";
                sfd.FilterIndex = 1;
                sfd.Title = "名前を付けてスキーマの保存";
                sfd.RestoreDirectory = true;
                sfd.OverwritePrompt = true;
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    TemplateUtil.WritePluginText(sfd.FileName);
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }


        /// <summary>
        /// メニューの終了
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmExit_Click(object sender, EventArgs e)
        {
            try
            {
                this.Close();
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// 設定ファイルチェック
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmTest_Click(object sender, EventArgs e)
        {
            try
            {
                if (String.IsNullOrEmpty(cmbConfig.Text))
                    return;

                StringBuilder err = ConfigTestUtil.Test(new ConfigFactory(cmbConfig.Text));
                if (err.Length > 0)
                {
                    txtConsole.Text = err.ToString();
                    throw new PlaggerException(String.Format("設定ファイルに問題点があります。\n{0}", err.ToString()));
                }

                MessageBox.Show("設定ファイルに問題はありません。", "インフォメッセージ", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// プラグイン説明のすべてを出力
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmPluginDesc_Click(object sender, EventArgs e)
        {
            try
            {
                tabControl.SelectedIndex = 2;

                string output = CommandExecute(@"CsharPlagger.exe", "/p");
                output = output.Replace("\n", "\r\n");
                OutputToConsole(output);
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// C#Plaggerの実行
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmStart_Click(object sender, EventArgs e)
        {
            try
            {
                if (dataGridView.RowCount < 1)
                    return;

                tabControl.SelectedIndex = 2;
#if DEBUG
                DebugRun();
#else
                string output = CommandExecute(@"CsharPlagger.exe", "/c:" + cmbConfig.Text);
                OutputToConsole(output);
#endif
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }


        private void DebugRun()
        {
            ConfigFactory factory = new ConfigFactory(cmbConfig.Text);
            StringBuilder err = ConfigTestUtil.Test(factory);
            if (err.Length > 0)
            {
                Console.Write(err.ToString());
                Environment.Exit(1);
            }

            // DBフレームワーク初期化
            if (factory.Plugins.DbeeConfig != null && factory.Plugins.DbeeConfig.useDB)
            {
                if (File.Exists(factory.Plugins.DbeeConfig.conf))
                {
                    ContextTrailer.GetInstance(factory.Plugins.DbeeConfig.conf);
                }
                else
                {
                    Console.Write("DB接続設定ファイルが見つかりません。");
                    logger.ErrorFormat("DB接続設定ファイルが見つかりません。{0}", factory.Plugins.DbeeConfig.conf);
                    return;
                }
            }

            // 設定されたプラグインの実行
            IDataContents data = new DataContentsImpl();

            foreach (PluginsModule module in factory.Plugins.module)
            {
                PluginExecutor executor = new PluginExecutor(factory);
                data = executor.Execute(data, module);
            }
        }

        /// <summary>
        /// 設定ファイルの編集可否
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmConfigEdit_Click(object sender, EventArgs e)
        {
            try
            {
                if (tsmConfigEdit.Checked)
                {
                    txtXml.ReadOnly = false;
                    txtXml.ForeColor = Color.Yellow;
                    tsmSaveConfig.Enabled = true;
                }
                else
                {
                    txtXml.ReadOnly = true;
                    txtXml.ForeColor = Color.White;
                    tsmSaveConfig.Enabled = false;
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// 設定ファイルの保存
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tsmSaveConfig_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog sfd = new SaveFileDialog();

                sfd.FileName = Path.GetFileName(cmbConfig.Text);
                sfd.InitialDirectory = @"";
                sfd.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
                sfd.FilterIndex = 1;
                sfd.Title = "名前を付けて設定ファイルの保存";
                sfd.RestoreDirectory = true;
                sfd.OverwritePrompt = true;
                sfd.CheckPathExists = true;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    XmlUtil.WriteXml(sfd.FileName, txtXml.Text);
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// ファイルダイアログ表示ボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnConfig_Click(object sender, EventArgs e)
        {
            try
            {
                OpenConfigFile();
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// 設定ファイルコンボ変更時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cmbConfig_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                UpdateDisplay();
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// グリッドの選択変更時
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dataGridView_SelectionChanged(object sender, EventArgs e)
        {
            try
            {
                if (dataGridView.RowCount > 0 && dataGridView.SelectedRows.Count > 0)
                {
                    int row = dataGridView.SelectedRows[0].Index;

                    IList list = (IList)dataGridView.DataSource;
                    ModuleInfo info = (ModuleInfo)list[row];
                    IPlugin plugin = info.PluginInstance;
                    InjectionUtil.Injection(info.Module, plugin);

                    propertyGrid.SelectedObject = plugin;

                    string usage = ShowUsageUtil.GetUsage("Bl.CsharPlagger.Plugin." + dataGridView[0, row].Value.ToString()).ToString();
                    txtConsole.Text = usage.Replace("\n", "\r\n");

                    toolStripStatusLabel1.Text = dataGridView[0, row].Value.ToString();
                    toolStripStatusLabel2.Text = dataGridView[1, row].Value.ToString();
                }
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// フォーム終了前
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormPlagger_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                ArrayList list = new ArrayList();
                foreach (string item in cmbConfig.Items)
                {
                    list.Add(item);
                }
                Settings.Default.ConfigList = list;
                Settings.Default.Save();
            }
            catch (PluginException ex)
            {
                RaiseError(ex);
            }
            catch (PlaggerException ex)
            {
                RaiseError(ex);
            }
        }

        /// <summary>
        /// 画面の更新
        /// </summary>
        private void UpdateDisplay()
        {
            if (!String.IsNullOrEmpty(cmbConfig.Text))
            {
                if (File.Exists(cmbConfig.Text))
                {
                    txtXml.Text = XmlUtil.ReadXml(cmbConfig.Text);
                    LoadConfig();
                }
                else
                {
                    MessageBox.Show("指定した設定ファイルは存在しません。", "警告メッセージ", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    cmbConfig.SelectAll();
                    cmbConfig.Focus();
                }
            }
        }

        /// <summary>
        /// 設定の読み込み
        /// </summary>
        private void LoadConfig()
        {
            dataGridView.DataSource = null;
            propertyGrid.SelectedObject = null;
            txtConsole.Text = "";

            StringBuilder err = ConfigTestUtil.Test(new ConfigFactory(cmbConfig.Text));
            if (err.Length > 0)
            {
                txtConsole.Text = err.ToString();
                throw new PlaggerException(String.Format("設定ファイルに問題点があります。\n{0}", err.ToString()));            
            }
            dataGridView.DataSource = PluginUtil.LoadConfig(cmbConfig.Text);
        }

        private void RaiseError(Exception ex)
        {
            logger.Error(ex.Message, ex);
            MessageBox.Show(ex.Message, ex.Source, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        private void OpenConfigFile()
        {
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.FileName = "Config.xml";
            ofd.InitialDirectory = @"";
            ofd.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.Title = "設定ファイルを選択してください";
            ofd.RestoreDirectory = true;
            ofd.CheckFileExists = true;
            ofd.CheckPathExists = true;

            if (ofd.ShowDialog() == DialogResult.OK)
            {
                bool isChangeConfig = false;

                if (cmbConfig.Text != ofd.FileName)
                    isChangeConfig = true;

                cmbConfig.Text = ofd.FileName;
                
                if (!cmbConfig.Items.Contains(ofd.FileName))
                {
                    cmbConfig.Items.Add(ofd.FileName);
                }

                if (!isChangeConfig)
                    LoadConfig();
            }
        }

        /// <summary>
        /// コマンドの実行
        /// </summary>
        /// <param name="command"></param>
        /// <param name="arg"></param>
        /// <returns></returns>
        private string CommandExecute(string command, string arg)
        {
            using (Process p = new Process())
            {
                p.StartInfo.FileName = command;
                p.StartInfo.CreateNoWindow = true;
                p.StartInfo.UseShellExecute = false;
                p.StartInfo.RedirectStandardOutput = true;
                p.StartInfo.Arguments = arg;

                p.Start();
                return p.StandardOutput.ReadToEnd();
            }
        }

        /// <summary>
        /// コンソールに出力
        /// </summary>
        /// <param name="s"></param>
        private void OutputToConsole(string s)
        {
            txtConsole.Text = s.Replace("\r\r\n", "\n");
        }

    }
}
