﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using Bl.CsharPlagger.Win;
using log4net;
using System.Reflection;
using System.Net.NetworkInformation;
using System.Threading;

namespace WinCsharPlagger
{
    static class Program
    {
        private static readonly ILog logger = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// アプリケーションのメイン エントリ ポイントです。
        /// </summary>
        [STAThread]
        static void Main()
        {
            NetworkChange.NetworkAvailabilityChanged += new NetworkAvailabilityChangedEventHandler(NetworkChange_NetworkAvailabilityChanged);

            Mutex mutex = null;
            try
            {
                mutex = GetMutex();
                if (isStarting(mutex))
                {
                    Application.EnableVisualStyles();
                    Application.SetCompatibleTextRenderingDefault(false);
                    PlaggerContext context = new PlaggerContext(new FormPlagger());
                    Application.Run(context);
                }
                else
                {
                    MessageBox.Show("アプリケーションは既に起動されています。", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
            catch (Exception ex)
            {
                logger.Error(ex.Message, ex);
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            finally
            {
                if (mutex != null)
                {
                    GC.KeepAlive(mutex);
                    mutex.Close();
                }
            }
        }

        /// <summary>
        /// ネットワーク障害発生
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        static void NetworkChange_NetworkAvailabilityChanged(object sender, NetworkAvailabilityEventArgs e)
        {
            if (e.IsAvailable)
                return;

            Exception ex = new Exception("ネットワーク障害が発生しています。");
            logger.Error(ex.Message, ex);

            MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        }

        /// <summary>
        /// Mutexの取得
        /// </summary>
        /// <returns></returns>
        static Mutex GetMutex()
        {
            Mutex mutex;
            OperatingSystem os = Environment.OSVersion;
            if (os.Platform == PlatformID.Win32NT && os.Version.Major >= 5)
            {
                mutex = new Mutex(false, @"Global\" + Application.ProductName);
            }
            else
            {
                mutex = new Mutex(false, Application.ProductName);
            }

            return mutex;
        }

        /// <summary>
        /// 2重起動チェック
        /// </summary>
        /// <param name="mutex"></param>
        /// <returns></returns>
        static bool isStarting(Mutex mutex)
        {
            return mutex.WaitOne(0, false);
        }

    }
}
